package com.yihu.hos.web.framework.baseservice;

import com.yihu.hos.web.framework.dao.SQLGeneralDAO;
import com.yihu.hos.web.framework.query.URLHqlQueryParser;
import org.hibernate.Criteria;
import org.hibernate.FlushMode;
import org.hibernate.criterion.Projections;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.text.ParseException;
import java.util.List;

public class BaseService<T> extends SQLGeneralDAO{

    /**
     * 查询列表(不分页)
     * @param filters
     * @return
     * @throws ParseException
     */
    public List search(String filters,String sorts) throws ParseException {
        return search(null,filters,sorts,null,null);
    }

    /**
     * 查询列表(不分页)
     * @param filters
     * @return
     * @throws ParseException
     */
    public List search(String filters) throws ParseException {
        return search(null,filters,null,null,null);
    }

    /**
     * 查询列表(不分页)
     * @param fields
     * @param filters
     * @param sorts
     * @return
     * @throws ParseException
     */
    public List search(String fields, String filters, String sorts) throws ParseException {
        return search(fields,filters,sorts,null,null);
    }

    /**
     * 查询列表(page,size都不为空,为分页查询)
     * @param fields
     * @param filters
     * @param sorts
     * @param page
     * @param size
     * @return
     * @throws ParseException
     */
    public List search(String fields, String filters, String sorts, Integer page, Integer size) throws ParseException {
        URLHqlQueryParser queryParser = createQueryParser(fields, filters, sorts);
        Criteria criteria = queryParser.makeCriteriaQuery();
        if(page!=null&&size!=null){
            criteria.setFirstResult((page - 1) * size).setMaxResults(size);
        }
        return criteria.list();
    }

    /**
     * 获取总条数
     * @param filters
     * @return
     * @throws ParseException
     */
    public long getCount(String filters) throws ParseException {
        URLHqlQueryParser queryParser = createQueryParser(null, filters, null);
        Criteria criteria = queryParser.makeCriteriaQuery();
        Criteria criteria1 = criteria.setProjection(Projections.rowCount());
        Long i = Long.parseLong(criteria1.uniqueResult().toString());
        return i;
    }

    private <T> URLHqlQueryParser createQueryParser(String fields, String filters, String orders) {
        if("\"\"".equals(fields)){
            fields=null;
        }
        if("\"\"".equals(filters)){
            filters="";
        }

        if("\"\"".equals(orders)){
            orders="";
        }
        URLHqlQueryParser queryParser = new URLHqlQueryParser<T>(fields, filters, orders)
                .setSession(getCurrentSession())
                .setEntityClass(getEntityClass());
        return queryParser;
    }

    public Class<T> getEntityClass() {
        Type genType = this.getClass().getGenericSuperclass();
        Type[] parameters = ((ParameterizedType) genType).getActualTypeArguments();
        return (Class) parameters[0];
    }


    public int delete(Object[] ids) {
        return delete(ids, getEntityClass());
    }
    public int delete(Object id) {
        Object[] obj;
        //判断是否是 List ,如果是list,则封装成obj数组在删除,
        if(id instanceof java.util.List){
            List idList =(List)id;
            obj = new Object[idList.size()];
            obj= idList.toArray();
        }else{
            obj = new Object[1];
            obj[0] = id;
        }
        return delete(obj, getEntityClass());
    }

    public int delete(Object[] ids, Class entity) {
        if(ids==null || ids.length==0)
            return 0;
        String hql =
                "DELETE FROM "+ entity.getSimpleName() +
                        " WHERE " + sessionFactory().getClassMetadata(entity).getIdentifierPropertyName() + " in(:ids) ";
        return currentSession().createQuery(hql)
                .setParameterList("ids", ids)
                .executeUpdate();
    }

    public T retrieve(Serializable id, Class entityClass) {
        return (T) currentSession().get(entityClass, id);
    }

    public T retrieve(Serializable id) {
        return (T) currentSession().get(getEntityClass(), id);
    }

    public List findAll(){

        return findAll(getEntityClass());
    }

    public List findAll(Class entityClass){
        currentSession().setFlushMode(FlushMode.MANUAL);
        return currentSession().createCriteria(entityClass).list();
    }

}
