package com.yihu.hos.web.framework.dao;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.JsonNodeType;
import com.yihu.hos.core.datatype.NumberUtil;
import com.yihu.hos.core.datatype.StringUtil;
import com.yihu.hos.core.log.Logger;
import com.yihu.hos.core.log.LoggerFactory;
import com.yihu.hos.web.framework.model.DataGridResult;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.standard.AdapterVersion;
import com.yihu.hos.web.framework.model.standard.StandardVersion;
import com.yihu.hos.web.framework.util.sql.SqlCreator;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.*;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.orm.hibernate5.HibernateTemplate;
import org.springframework.orm.hibernate5.LocalSessionFactoryBean;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import javax.persistence.Table;
import java.io.IOException;
import java.io.Serializable;
import java.math.BigInteger;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.*;

/**
 * @author HZY
 * @vsrsion 1.0
 * Created at 2016/8/8.
 */
@Transactional
@Repository("sqlGeneralDAO")
public class SQLGeneralDAO implements XSQLGeneralDAO {
    static private final Logger logger = LoggerFactory.getLogger(SQLGeneralDAO.class);

    @Autowired
    protected LocalSessionFactoryBean localSessionFactoryBean;
    @Autowired
    protected ObjectMapper objectMapper;
    @Autowired
    protected JdbcTemplate jdbcTemplate;
    @Autowired
    protected HibernateTemplate hibernateTemplate;

    public SQLGeneralDAO() {
    }

    protected SessionFactory sessionFactory() {
        return localSessionFactoryBean.getObject();
    }

    protected Session currentSession() {
        return localSessionFactoryBean.getObject().getCurrentSession();
    }

    public JdbcTemplate getJdbcTemplate() {
        return this.jdbcTemplate;
    }

    public HibernateTemplate getHibernateTemplate() {
        return this.hibernateTemplate;
    }

    public void beginTransaction() throws Exception {
        this.hibernateTemplate.getSessionFactory().getCurrentSession().getTransaction().begin();
    }

    public void commitTransaction() throws Exception {
        this.hibernateTemplate.getSessionFactory().getCurrentSession().getTransaction().commit();
    }

    public Session openSession() {
        return this.getHibernateTemplate().getSessionFactory().openSession();
    }

    public Session getCurrentSession() {
        return this.getHibernateTemplate().getSessionFactory().getCurrentSession();
    }

    public String getTableName(Class<?> clazz) {
        Table table = (Table) clazz.getAnnotation(Table.class);
        return table.name();
    }

    public DataGridResult getDataGridResult(String hql, Integer page, Integer rows) {
        DataGridResult dataGridResult = new DataGridResult();
        Query query = this.hibernateTemplate.getSessionFactory().getCurrentSession().createQuery(hql);
        if (page != null && rows != null) {
            dataGridResult.setPageSize(rows.intValue());
            dataGridResult.setCurrPage(page.intValue());
            dataGridResult.setTotalCount(query.list().size());
            query.setMaxResults(rows.intValue());
            query.setFirstResult((page.intValue() - 1) * rows.intValue());
            dataGridResult.setDetailModelList(query.list());
        } else {
            List list = query.list();
            dataGridResult.setDetailModelList(list);
            dataGridResult.setTotalCount(list.size());
        }

        return dataGridResult;
    }

    public DataGridResult getDataGridResult(Query query, Integer page, Integer rows) {
        DataGridResult dataGridResult = new DataGridResult();
        if (page != null && rows != null) {
            dataGridResult.setPageSize(rows.intValue());
            dataGridResult.setCurrPage(page.intValue());
            dataGridResult.setTotalCount(query.list().size());
            query.setMaxResults(rows.intValue());
            query.setFirstResult((page.intValue() - 1) * rows.intValue());
            dataGridResult.setDetailModelList(query.list());
        } else {
            List list = query.list();
            dataGridResult.setDetailModelList(list);
            dataGridResult.setTotalCount(list.size());
        }

        return dataGridResult;
    }

    public <T> T saveEntity(T entity) throws Exception {
        if (entity != null) {
            this.hibernateTemplate.save(entity);
        }
        return entity;
    }

    public <T> T updateEntity(T entity) throws Exception {
        if (entity != null) {
            this.hibernateTemplate.update(entity);
        }
        return entity;
    }

    public void saveOrUpdateEntity(Object entity) throws Exception {
        if (entity != null) {
            this.hibernateTemplate.saveOrUpdate(entity);
        }
    }

    public void mergeEntity(Object entity) throws Exception {
        if (entity != null) {
            this.hibernateTemplate.merge(entity);
        }
    }

    public void deleteEntity(Object entity) throws Exception {
        if (entity != null) {
            this.hibernateTemplate.delete(entity);
        }
    }

    public <T> void deleteEntity(Class<T> cls, Serializable id) throws Exception {
        Object obj = this.hibernateTemplate.load(cls, id);
        this.hibernateTemplate.delete(obj);
    }

    public <T> T getEntity(Class<T> cls, Serializable id) throws Exception {
        return this.hibernateTemplate.get(cls, id);
    }

    public List<Map<String, Object>> queryListBySql(String sql) throws Exception {
        List result = this.jdbcTemplate.queryForList(sql);
        return result;
    }

    public List<Map<String, Object>> queryListBySql(String sql, List params) throws Exception {
        List result = this.jdbcTemplate.queryForList(sql, params.toArray());
        return result;
    }

    public <T> List<T> queryListBySql(String sql, Class obj) throws Exception {
        RowMapper<T> rowMapper = BeanPropertyRowMapper.newInstance(obj);
        List result = this.jdbcTemplate.query(sql, rowMapper);
        return result;
    }

    public <T> List<T> queryListBySql(String sql, List params, Class obj) throws Exception {
        RowMapper<T> rowMapper = BeanPropertyRowMapper.newInstance(obj);
        List result = this.jdbcTemplate.query(sql, params.toArray(), rowMapper);
        return result;
    }

    public <T> T queryObjBySql(String sql, List params, Class obj) throws Exception {
        RowMapper<T> rowMapper = BeanPropertyRowMapper.newInstance(obj);
        T result = this.jdbcTemplate.queryForObject(sql, params.toArray(), rowMapper);
        return result;
    }

    public <T> T queryObjBySql(String sql, Class obj) throws Exception {
        RowMapper<T> rowMapper = BeanPropertyRowMapper.newInstance(obj);
        T result = this.jdbcTemplate.queryForObject(sql, rowMapper);
        return result;
    }

    public <T> T scalarBySql(String sql, Class cls) throws Exception {
        T result = (T) this.jdbcTemplate.queryForObject(sql, cls);
        return result;
    }

    public void execute(String sql) throws Exception {
        this.jdbcTemplate.execute(sql);
    }

    public <T> List getEntityList(Class<T> cls, String hql) {
        Query query = this.getCurrentSession().createQuery(hql);
        return query.list();
    }

    public <T> List getEntityList(Class<T> cls, String condition, String order, Integer limit, Integer offset) throws IOException {
        Session session = this.getCurrentSession();
        Criteria criteria = session.createCriteria(cls);
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode jsonNode;
        Iterator fieldNames;
        String fieldName;
        if (!StringUtil.isEmpty(condition)) {
            jsonNode = objectMapper.readTree(condition);
            fieldNames = jsonNode.fieldNames();

            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                if (jsonNode.get(fieldName).isInt()) {
                    criteria.add(Restrictions.eq(fieldName, Integer.valueOf(jsonNode.get(fieldName).asInt())));
                } else {
                    criteria.add(Restrictions.eq(fieldName, jsonNode.get(fieldName).asText()));
                }
            }
        }

        if (!StringUtil.isEmpty(order)) {
            jsonNode = objectMapper.readTree(order);
            fieldNames = jsonNode.fieldNames();

            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                String value = jsonNode.get(fieldName).asText().toUpperCase();
                if (value.equals("ASC")) {
                    criteria.addOrder(Order.asc(fieldName));
                } else if (value.equals("DESC")) {
                    criteria.addOrder(Order.desc(fieldName));
                }
            }
        }

        if (limit != null) {
            criteria.setMaxResults(limit.intValue());
            if (offset != null) {
                criteria.setFirstResult((offset.intValue() - 1) * limit.intValue());
            }
        }

        return criteria.list();
    }

    public <T> Long getEntitysCount(Class<T> cls, String condition) throws IOException {
        Session session = this.getCurrentSession();
        Criteria criteria = session.createCriteria(cls);
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode jsonNode;
        Iterator fieldNames;
        String fieldName;
        if (!StringUtil.isEmpty(condition)) {
            jsonNode = objectMapper.readTree(condition);
            fieldNames = jsonNode.fieldNames();

            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                if (jsonNode.get(fieldName).isInt()) {
                    criteria.add(Restrictions.eq(fieldName, Integer.valueOf(jsonNode.get(fieldName).asInt())));
                } else {
                    criteria.add(Restrictions.eq(fieldName, jsonNode.get(fieldName).asText()));
                }
            }
        }

        Long size = 0l;
        if (criteria.list() != null && criteria.list().size() > 0) {
            size = Long.valueOf(criteria.list().size());
        }

        return size;
    }

    public <T> List getEntityListByParentId(Class<T> cls, String parentFiledName, Long patentId, String condition, String order, Integer limit, Integer offset) throws IOException {
        Session session = this.getCurrentSession();
        Criteria criteria = session.createCriteria(cls);
        criteria.add(Restrictions.eq(parentFiledName, patentId));
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode jsonNode;
        Iterator fieldNames;
        String fieldName;
        if (!StringUtil.isEmpty(condition)) {
            jsonNode = objectMapper.readTree(condition);
            fieldNames = jsonNode.fieldNames();

            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                if (jsonNode.get(fieldName).isInt()) {
                    criteria.add(Restrictions.eq(fieldName, Integer.valueOf(jsonNode.get(fieldName).asInt())));
                } else {
                    criteria.add(Restrictions.eq(fieldName, jsonNode.get(fieldName).asText()));
                }
            }
        }

        if (!StringUtil.isEmpty(order)) {
            jsonNode = objectMapper.readTree(order);
            fieldNames = jsonNode.fieldNames();

            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                String value = jsonNode.get(fieldName).asText().toUpperCase();
                if (value.equals("ASC")) {
                    criteria.addOrder(Order.asc(fieldName));
                } else if (value.equals("DESC")) {
                    criteria.addOrder(Order.desc(fieldName));
                }
            }
        }

        if (limit != null) {
            criteria.setMaxResults(limit.intValue());
            if (offset != null) {
                criteria.setFirstResult((offset.intValue() - 1) * limit.intValue());
            }
        }

        return criteria.list();
    }

    public <T> List getEntityListByParentId(Class<T> cls, String parentFiledName, Integer patentId, String condition, String order, Integer limit, Integer offset) throws IOException {
        Session session = this.getCurrentSession();
        Criteria criteria = session.createCriteria(cls);
        criteria.add(Restrictions.eq(parentFiledName, patentId));
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode jsonNode;
        Iterator fieldNames;
        String fieldName;
        if (!StringUtil.isEmpty(condition)) {
            jsonNode = objectMapper.readTree(condition);
            fieldNames = jsonNode.fieldNames();

            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                if (jsonNode.get(fieldName).isInt()) {
                    criteria.add(Restrictions.eq(fieldName, Integer.valueOf(jsonNode.get(fieldName).asInt())));
                } else {
                    criteria.add(Restrictions.eq(fieldName, jsonNode.get(fieldName).asText()));
                }
            }
        }

        if (!StringUtil.isEmpty(order)) {
            jsonNode = objectMapper.readTree(order);
            fieldNames = jsonNode.fieldNames();

            while (fieldNames.hasNext()) {
                fieldName = (String) fieldNames.next();
                String value = jsonNode.get(fieldName).asText().toUpperCase();
                if (value.equals("ASC")) {
                    criteria.addOrder(Order.asc(fieldName));
                } else if (value.equals("DESC")) {
                    criteria.addOrder(Order.desc(fieldName));
                }
            }
        }

        if (limit != null) {
            criteria.setMaxResults(limit.intValue());
            if (offset != null) {
                criteria.setFirstResult((offset.intValue() - 1) * limit.intValue());
            }
        }

        return criteria.list();
    }

    public List getList(Class tClass, String tableName, String condition, String order, Integer limit, Integer offset) {
        SqlCreator sqlCreator = new SqlCreator(tClass);
        ObjectMapper objectMapper = new ObjectMapper();

        try {
            JsonNode sql;
            Iterator query;
            String fieldName;
            String value;
            if (!StringUtil.isEmpty(condition)) {
                sql = objectMapper.readTree(condition);
                query = sql.fieldNames();

                while (query.hasNext()) {
                    fieldName = (String) query.next();
                    value = sql.get(fieldName).asText();
                    if (sql.get(fieldName).getNodeType().equals(JsonNodeType.NUMBER)) {
                        sqlCreator.equalCondition(fieldName, Integer.valueOf(Integer.parseInt(value)));
                    } else {
                        sqlCreator.equalCondition(fieldName, value);
                    }
                }
            }

            if (!StringUtil.isEmpty(order)) {
                sql = objectMapper.readTree(order);
                query = sql.fieldNames();

                while (query.hasNext()) {
                    fieldName = (String) query.next();
                    value = sql.get(fieldName).asText().toUpperCase();
                    sqlCreator.order(fieldName, value);
                }
            }
        } catch (Exception var13) {
            var13.printStackTrace();
        }

        String sql1 = sqlCreator.selectData(tableName);
        Query query1 = this.getQuery(sqlCreator, sql1);
        if (limit != null) {
            query1.setMaxResults(limit.intValue());
            if (offset != null) {
                query1.setFirstResult(offset.intValue() * limit.intValue());
            }
        }

        return query1.list();
    }

    public List getListByIdList(Class tClass, String tableName, String fieldName, List idList) {
        SqlCreator sqlCreator = new SqlCreator(tClass);

        try {
            sqlCreator.inCondition(fieldName, idList);
        } catch (Exception var8) {
            var8.printStackTrace();
            return null;
        }

        String sql = sqlCreator.selectData(tableName);
        Query query = this.getQuery(sqlCreator, sql);
        return query.list();
    }


    public Integer getDataSetInt(Class tClass, String tableName, String condition) {
        SqlCreator sqlCreator = new SqlCreator(tClass);
        ObjectMapper objectMapper = new ObjectMapper();

        try {
            if (!StringUtil.isEmpty(condition)) {
                JsonNode sql = objectMapper.readTree(condition);
                Iterator query = sql.fieldNames();

                while (query.hasNext()) {
                    String count = (String) query.next();
                    String value = sql.get(count).asText();
                    if (NumberUtil.isInteger(value)) {
                        sqlCreator.equalCondition(count, Integer.valueOf(Integer.parseInt(value)));
                    } else {
                        sqlCreator.equalCondition(count, value);
                    }
                }
            }
        } catch (Exception var10) {
            var10.printStackTrace();
        }

        String sql1 = sqlCreator.countData(tableName);
        Query query1 = this.getQuery(sqlCreator, sql1);
        Integer count1 = Integer.valueOf(Integer.parseInt(StringUtil.toString(query1.list().get(0))));
        return count1;
    }

    public Object get(Class tClass, String tableName, Object id) {
        SqlCreator sqlCreator = new SqlCreator(tClass);
        sqlCreator.equalCondition("id", id);
        String sql = sqlCreator.selectData(tableName);
        Query query = this.getQuery(sqlCreator, sql);
        return query.uniqueResult();
    }

    public Object findByfield(Class tClass, String tableName, String fieldName, Object id) {
        SqlCreator sqlCreator = new SqlCreator(tClass);

        try {
            sqlCreator.inCondition(fieldName, id);
        } catch (Exception var8) {
            var8.printStackTrace();
            return null;
        }

        String sql = sqlCreator.selectData(tableName);
        Query query = this.getQuery(sqlCreator, sql);
        return query.uniqueResult();
    }

    public Query getQuery(SqlCreator sqlCreator, String sql) {
        SQLQuery query = this.getCurrentSession().createSQLQuery(sql);
        Iterator var4 = sqlCreator.getKeyValueMap().keySet().iterator();

        while (var4.hasNext()) {
            String key = (String) var4.next();
            Object value = sqlCreator.getKeyValueMap().get(key);
            if (value instanceof Collection) {
                query.setParameterList(key, (Collection) value);
            } else {
                query.setParameter(key, value);
            }
        }

        query.setResultTransformer(sqlCreator.getTransformer());
        return query;
    }

    public void insertBatch(final List<String> insertSqlList) {
//        Session session = this.getHibernateTemplate().getSessionFactory().getCurrentSession();
        String[] sqlStr = new String[insertSqlList.size()];
        String[] sqlArr = insertSqlList.toArray(sqlStr);
        this.jdbcTemplate.batchUpdate(sqlArr);

    }

    public Integer getMaxId(String tableName) {
        String sql = "select max(id) from " + tableName;
        SQLQuery query = this.getCurrentSession().createSQLQuery(sql);
        Object object = query.uniqueResult();
        Integer maxId = Integer.valueOf(object == null ? 1 : Integer.parseInt(object.toString()) + 1);
        return maxId;
    }

    public Query getExeuteQuery(SqlCreator sqlCreator, String sql) {
        SQLQuery query = this.getCurrentSession().createSQLQuery(sql);
        Iterator var4 = sqlCreator.getKeyValueMap().keySet().iterator();

        while (var4.hasNext()) {
            String key = (String) var4.next();
            Object value = sqlCreator.getKeyValueMap().get(key);
            if (value instanceof Collection) {
                query.setParameterList(key, (Collection) value);
            } else {
                query.setParameter(key, value);
            }
        }

        return query;
    }

    public List findInList(String hql , String[] params , Object[] values) {
        Query query = this.getCurrentSession().createQuery(hql);
        if (params == null || values == null){
            return null;
        }

        for (int i=0;i <params.length;i++){
            if (values[i] instanceof Collection) {
                query.setParameterList(params[i], (Collection) values[i]);
            } else {
                query.setParameter(params[i], values[i]);
            }
        }
        return query.list();
    }

    public Boolean tableIsExist(String sql) {
        BigInteger tableNum = (BigInteger) this.getCurrentSession().createSQLQuery(sql).uniqueResult();
        if (tableNum.intValue() > 0) {
            return true;
        }
        return false;
    }

    protected void doClose(Session session, Statement stmt, ResultSet rs) {
        if (rs != null) {
            try {
                rs.close();
                rs = null;
            } catch (Exception var6) {
                rs = null;
                logger.error("close-ResultSet ", var6);
                var6.printStackTrace();
            }
        }

        if (stmt != null) {
            try {
                stmt.close();
                stmt = null;
            } catch (Exception var5) {
                stmt = null;
                logger.error("close-ResultSet ", var5);
                var5.printStackTrace();
            }
        }

    }

    //region 标准相关

    public List searchVersionData(Class tClass, String stdVersion,String dataType, String condition) throws IOException {
        return searchVersionData(tClass,stdVersion,dataType,condition,null,null,null);
    }

    public List searchVersionData(Class tClass, String stdVersion,String dataType, String condition, String order, Integer page, Integer size) throws IOException {
        String tableName = null;
        if (!StringUtil.isEmpty(stdVersion)) {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            tableName = standardVersion.getTableName(dataType);
        } else {
            tableName = getTableName(tClass);
        }

        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(condition)) {
            String[] filterArray = condition.split(";");
            for (int i = 0; i < filterArray.length; ++i) {
                String[] tokens = filterArray[i].split("=");
                if (tokens.length > 2) {
                    for (int j = 1; j < tokens.length; j++) {
                        if (j == tokens.length - 1)
                            tokens[1] = tokens[j];
                        else
                            tokens[0] += " " + tokens[j];
                    }
                }
//            if (tokens.length > 2) throw new IllegalArgumentException("无效过滤参数");
                String group = null;
                if (tokens.length >= 2) group = tokens[1];

                if (group == null)
                    group = Integer.toString(i);

                sqlCreator.likeCondition(tokens[0], tokens[1]);
            }
        }

        //order
        if (StringUtils.isNotEmpty(order)) {
            String[] orderArray = order.split(",");
            for (String elem : orderArray) {
                String value = elem.startsWith("+") ? "ASC" : "DESC";
                sqlCreator.order(elem.substring(1), value);
            }
        }
        String sql = sqlCreator.selectData(tableName);
        Query query = getQuery(sqlCreator, sql);
        if (size != null) {
            query.setMaxResults(size);
            if (page != null) {
                query.setFirstResult((page - 1) * size);
            }
        }
        return query.list();
    }

    public Long searchVersionCount(Class tClass, String stdVersion, String dataType, String condition) throws IOException {
        String tableName = null;
        if (!StringUtil.isEmpty(stdVersion)) {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            tableName = standardVersion.getTableName(dataType);
        } else {
            tableName = getTableName(tClass);
        }

        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(condition)) {
            String[] filterArray = condition.split(";");
            for (int i = 0; i < filterArray.length; ++i) {
                String[] tokens = filterArray[i].split("=");
                if (tokens.length > 2) {
                    for (int j = 1; j < tokens.length; j++) {
                        if (j == tokens.length - 1)
                            tokens[1] = tokens[j];
                        else
                            tokens[0] += " " + tokens[j];
                    }
                }
//            if (tokens.length > 2) throw new IllegalArgumentException("无效过滤参数");
                String group = null;
                if (tokens.length >= 2) group = tokens[1];

                if (group == null)
                    group = Integer.toString(i);

                sqlCreator.likeCondition(tokens[0], tokens[1]);
            }
        }

        String sql = sqlCreator.countData(tableName);
        Query query = getCurrentSession().createSQLQuery(sql);
        for (String key : sqlCreator.getKeyValueMap().keySet()) {
            query.setParameter(key, sqlCreator.getKeyValueMap().get(key));
        }
        Long count = Long.valueOf(StringUtil.toString(query.list().get(0)));
        return count;
    }

    public Long searchCount(Class tClass, String stdVersion, String dataType, String condition) throws IOException {
        String tableName = null;
        if (!StringUtil.isEmpty(stdVersion)) {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            tableName = standardVersion.getTableName(dataType);
        } else {
            tableName = getTableName(tClass);
        }

        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(condition)) {
            Map<String, List<String>> whereMap = new HashMap<>();

            String[] filterArray = condition.split(";");
            for (int i = 0; i < filterArray.length; ++i) {
                String[] tokens = filterArray[i].split(" ");
                if (tokens.length > 2) {
                    for (int j = 1; j < tokens.length; j++) {
                        if (j == tokens.length - 1)
                            tokens[1] = tokens[j];
                        else
                            tokens[0] += " " + tokens[j];
                    }
                }

                String group = null;
                if (tokens.length >= 2)
                    group = tokens[1];
                if (group == null)
                    group = Integer.toString(i);

                List<String> list = whereMap.get(group);
                if(list==null || list.size()==0){
                    list = new ArrayList<String>();
                    list.add(tokens[0]);
                }else{
                    list.add(tokens[0]);
                }
                whereMap.put(group,list);
            }

            Collection<List<String>> lists = whereMap.values();

            for(List<String> whereCondition :lists){
                sqlCreator.makeWhere(whereCondition);
            }
        }

        String sql = sqlCreator.countData(tableName);
        Query query = getCurrentSession().createSQLQuery(sql);
        for (String key : sqlCreator.getKeyValueMap().keySet()) {
            query.setParameter(key, sqlCreator.getKeyValueMap().get(key));
        }
        Long count = Long.valueOf(StringUtil.toString(query.list().get(0)));
        return count;
    }

    //这边不支持分组条件查询,如需要分组查询
    //因为调用的地方太多,未做大改造,新增了  searchPage 方法
    public List searchPageData(Class tClass,String dataType, String stdVersion, String condition, String order, Integer offset, Integer limit) throws IOException {
        StandardVersion standardVersion = new StandardVersion(stdVersion);
        String tableName = standardVersion.getTableName(dataType);
        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(condition)) {
            String[] filterArray = condition.split(";");
            for (int i = 0; i < filterArray.length; ++i) {
                String[] tokens = filterArray[i].split("=");
                if (tokens.length > 2){
                    for(int j=1; j<tokens.length; j++){
                        if(j==tokens.length-1)
                            tokens[1] = tokens[j];
                        else
                            tokens[0] += " " + tokens[j] ;
                    }
                }
//            if (tokens.length > 2) throw new IllegalArgumentException("无效过滤参数");
                String group = null;
                if (tokens.length >= 2) group = tokens[1];

                if (group == null)
                    group = Integer.toString(i);

                if(tokens[1].contains(",")){
                    sqlCreator.inCondition(tokens[0],tokens[1]);
                }else{
                    sqlCreator.equalConditionQue(tokens[0],tokens[1]);
                }
            }
        }

        //order
        if (StringUtils.isNotEmpty(order)) {
            String[] orderArray = order.split(",");
            for(String elem : orderArray){
                String value = elem.startsWith("-") ? "DESC" : "ASC";
                sqlCreator.order(elem.substring(1), value);
            }
        }
        String sql = sqlCreator.selectData(tableName);
        Query query = getQuery(sqlCreator, sql);
        if (limit != null) {
            query.setMaxResults(limit);
            if (offset != null) {
                query.setFirstResult((offset - 1) * limit);
            }
        }
        return query.list();
    }



    public List searchPage(Class tClass,String dataType, String version, String condition, String order, Integer offset, Integer limit,boolean isStandard) throws IOException {
        String tableName;
        if(isStandard){
            StandardVersion standardVersion = new StandardVersion(version);
            tableName = standardVersion.getTableName(dataType);
        }else{
            AdapterVersion adapterVersion = new AdapterVersion(version);
            tableName = adapterVersion.getTableName(dataType);
        }

        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(condition)) {
            Map<String, List<String>> whereMap = new HashMap<>();

            String[] filterArray = condition.split(";");
            for (int i = 0; i < filterArray.length; ++i) {
                String[] tokens = filterArray[i].split(" ");
                if (tokens.length > 2) {
                    for (int j = 1; j < tokens.length; j++) {
                        if (j == tokens.length - 1)
                            tokens[1] = tokens[j];
                        else
                            tokens[0] += " " + tokens[j];
                    }
                }

                String group = null;
                if (tokens.length >= 2)
                    group = tokens[1];
                if (group == null)
                    group = Integer.toString(i);

                List<String> list = whereMap.get(group);
                if(list==null || list.size()==0){
                    list = new ArrayList<String>();
                    list.add(tokens[0]);
                }else{
                    list.add(tokens[0]);
                }
                whereMap.put(group,list);
            }

            Collection<List<String>> lists = whereMap.values();

            for(List<String> whereCondition :lists){
                sqlCreator.makeWhere(whereCondition);
            }
        }

        //order
        if (StringUtils.isNotEmpty(order)) {
            String[] orderArray = order.split(",");
            for(String elem : orderArray){
                String value = elem.startsWith("-") ? "DESC" : "ASC";
                sqlCreator.order(elem.substring(1), value);
            }
        }
        String sql = sqlCreator.selectData(tableName);
        Query query = getQuery(sqlCreator, sql);
        if (limit != null) {
            query.setMaxResults(limit);
            if (offset != null) {
                query.setFirstResult((offset - 1) * limit);
            }
        }
        return query.list();
    }

    public boolean isExistByField(Class tClass, String stdVersion,String dataType,String field, Object val) throws IOException {

        return isExistByFields(
                tClass,
                stdVersion,
                dataType,
                new String[]{field},
                new Object[]{val}

        );
    }

    public boolean isExistByFields(Class tClass, String stdVersion,String dataType,String[] fields, Object[] vals) throws IOException {
        String tableName = null;
        if (!StringUtil.isEmpty(stdVersion)) {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            tableName = standardVersion.getTableName(dataType);
        } else {
            tableName = getTableName(tClass);
        }

        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(fields) && !StringUtil.isEmpty(vals)) {

            for (int i = 0; i < fields.length; ++i) {
                if(fields[i]!=null){
                    sqlCreator.equalConditionQue(fields[i], vals[i]);
                }
            }
        }

        String sql = sqlCreator.selectData(tableName);
        Query query = getQuery(sqlCreator, sql);
        Long count = Long.valueOf(StringUtil.toString(query.list().size()));
        return count>0;
    }

    public <T> T retrieve(Class<T> tclass,String tableName,Object id){
        return (T) get(tclass,tableName,id);
    }

    public Object get(Class tClass, String stdVersion, String dataType, String id) {
        try {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            if (id != null) {
                String tableName = standardVersion.getTableName(dataType);
                SqlCreator sqlCreator = new SqlCreator(tClass);
                sqlCreator.equalCondition("id", id);
                String sql = sqlCreator.selectData(tableName);
                Query query = getQuery(sqlCreator, sql);
                return query.uniqueResult();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Result.error("获取标准数据元失败");
    }

    public  <T> List<T> findByFields(Class<T> tClass, String stdVersion, String[] fields, Object[] vals) throws IOException {
        String tableName = null;
        if (!StringUtil.isEmpty(stdVersion)) {
            StandardVersion standardVersion = new StandardVersion(stdVersion);
            tableName = standardVersion.getDataSetTableName();
        } else {
            tableName = getTableName(tClass);
        }

        SqlCreator sqlCreator = new SqlCreator(tClass);
        //where
        if (!StringUtil.isEmpty(fields) && !StringUtil.isEmpty(vals)) {

            for (int i = 0; i < fields.length; ++i) {
                if(fields[i]!=null){
                    sqlCreator.likeCondition(fields[i], vals[i]);
                }
            }
        }

        String sql = sqlCreator.selectData(tableName);
        Query query = getQuery(sqlCreator, sql);
        return query.list();
    }

    public boolean delete(Class tclass, Object[] ids, String dataType, String version) {
        if (ids == null || ids.length == 0)
            return false;

        StandardVersion standardVersion = new StandardVersion(version);
        String tableName = standardVersion.getTableName(dataType);
        String idField = sessionFactory().getClassMetadata(tclass).getIdentifierPropertyName();
        String hql =
                "DELETE FROM " + tableName +
                        " WHERE " + idField + " in (:ids) ";

        Query query = this.getCurrentSession().createQuery(hql);
        query.setParameterList("ids", ids);

        return query.executeUpdate() > 0;

    }

    public boolean deleteByField(String field, Object[] ids, String dataType, String version) {
        if (ids == null || ids.length == 0)
            return false;

        StandardVersion standardVersion = new StandardVersion(version);
        String tableName = standardVersion.getTableName(dataType);
        String sql =
                "DELETE FROM " + tableName +
                        " WHERE " + field + " in (:ids) ";
        Query query = this.getCurrentSession().createSQLQuery(sql);
        query.setParameterList("ids", ids);
        return query.executeUpdate() > 0;

    }
}
